@@ -13,6 +13,7 @@
 #    under the License.
 
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 import pecan
 import six
 
@@ -20,9 +21,11 @@ from zun.api.controllers import base
 from zun.api.controllers import link
 from zun.api.controllers.v1 import collection
 from zun.api.controllers.v1.schemas import capsules as schema
+from zun.api.controllers.v1.schemas import parameter_types
 from zun.api.controllers.v1.views import capsules_view as view
 from zun.api import utils as api_utils
 from zun.api import validation
+from zun.api.validation import validators
 from zun.common import consts
 from zun.common import exception
 from zun.common.i18n import _
@@ -33,6 +36,7 @@ import zun.conf
 from zun import objects
 from zun.volume import cinder_api as cinder
 
+
 CONF = zun.conf.CONF
 LOG = logging.getLogger(__name__)
 
@@ -51,6 +55,40 @@ def check_policy_on_capsule(capsule, action):
     policy.enforce(context, action, capsule, action=action)
 
 
+def check_capsule_template(tpl):
+    # TODO(kevinz): add volume spec check
+    tpl_json = tpl
+    if isinstance(tpl, six.string_types):
+        try:
+            tpl_json = jsonutils.loads(tpl)
+        except Exception as e:
+            raise exception.FailedParseStringToJson(e)
+
+        validator = validators.SchemaValidator(
+            parameter_types.capsule_template)
+        validator.validate(tpl_json)
+
+    kind_field = tpl_json.get('kind')
+    if kind_field not in ['capsule', 'Capsule']:
+        raise exception.InvalidCapsuleTemplate("kind fields need to be "
+                                               "set as capsule or Capsule")
+
+    spec_field = tpl_json.get('spec')
+    if spec_field is None:
+        raise exception.InvalidCapsuleTemplate("No Spec found")
+    # Align the Capsule restartPolicy with container restart_policy
+    # Also change the template filed name from Kubernetes type to OpenStack
+    # type.
+    if 'restartPolicy' in spec_field.keys():
+        spec_field['restartPolicy'] = \
+            utils.VALID_CAPSULE_RESTART_POLICY[spec_field['restartPolicy']]
+        spec_field[utils.VALID_CAPSULE_FIELD['restartPolicy']] = \
+            spec_field.pop('restartPolicy')
+    if spec_field.get('containers') is None:
+        raise exception.InvalidCapsuleTemplate("No valid containers field")
+    return spec_field, tpl_json
+
+
 class CapsuleCollection(collection.Collection):
     """API representation of a collection of Capsules."""
 
@@ -161,7 +199,7 @@ class CapsuleController(base.Controller):
         capsules_template = capsule_dict.get('template')
 
         spec_content, template_json = \
-            utils.check_capsule_template(capsules_template)
+            check_capsule_template(capsules_template)
 
         containers_spec, init_containers_spec = \
             utils.capsule_get_container_spec(spec_content)
@@ -209,8 +247,32 @@ class CapsuleController(base.Controller):
         new_capsule.image = CONF.sandbox_image
         new_capsule.image_driver = CONF.sandbox_image_driver
 
+        # calculate capsule cpu/ram
+        # 1. sum all cpu/ram of regular containers
+        for container_dict in containers_spec:
+            if not container_dict.get('resources'):
+                continue
+            allocation = container_dict['resources']['requests']
+            if allocation.get('cpu'):
+                capsule_need_cpu += allocation['cpu']
+            if allocation.get('memory'):
+                capsule_need_memory += allocation['memory']
+        # 2. take the maximum of each init container
+        for container_dict in init_containers_spec:
+            if not container_dict.get('resources'):
+                continue
+            allocation = container_dict['resources']['requests']
+            if allocation.get('cpu'):
+                capsule_need_cpu = max(capsule_need_cpu, allocation['cpu'])
+            if allocation.get('memory'):
+                capsule_need_memory = max(capsule_need_memory,
+                                          allocation['memory'])
+
         merged_containers_spec = init_containers_spec + containers_spec
         for container_spec in merged_containers_spec:
+            if container_spec.get('image_pull_policy'):
+                policy.enforce(context, "container:create:image_pull_policy",
+                               action="container:create:image_pull_policy")
             container_dict = container_spec
             container_dict['project_id'] = context.project_id
             container_dict['user_id'] = context.user_id
@@ -225,26 +287,29 @@ class CapsuleController(base.Controller):
                 container_dict['command'] = container_dict['args']
                 container_dict.pop('args')
 
-            # NOTE(kevinz): Don't support port remapping, will find a
-            # easy way to implement it.
-            # if container need to open some port, just open it in container,
-            # user can change the security group and getting access to port.
             if container_dict.get('ports'):
-                container_dict.pop('ports')
+                exposed_ports = {}
+                ports = container_dict.pop('ports')
+                for port in ports:
+                    container_port = "%s/%s" % (
+                        port['containerPort'],
+                        port.get('protocol', 'tcp').lower())
+                    host_port = {}
+                    exposed_ports[container_port] = host_port
+                container_dict['exposed_ports'] = exposed_ports
 
             if container_dict.get('resources'):
                 resources_list = container_dict.get('resources')
                 allocation = resources_list.get('requests')
                 if allocation.get('cpu'):
-                    capsule_need_cpu += allocation.get('cpu')
                     container_dict['cpu'] = allocation.get('cpu')
                 if allocation.get('memory'):
-                    capsule_need_memory += allocation.get('memory')
                     container_dict['memory'] = str(allocation['memory'])
                 container_dict.pop('resources')
 
+            container_dict['image_pull_policy'] = (
+                container_dict.get('image_pull_policy', 'always').lower())
             container_dict['status'] = consts.CREATING
-            container_dict['interactive'] = True
             container_dict['capsule_id'] = new_capsule.id
             container_dict['restart_policy'] = container_restart_policy
             if container_spec in init_containers_spec:
