@@ -26,6 +26,7 @@ from zun.api.controllers import base
 from zun.api.controllers import link
 from zun.api.controllers.v1 import collection
 from zun.api.controllers.v1.schemas import containers as schema
+from zun.api.controllers.v1.views import actions_view
 from zun.api.controllers.v1.views import containers_view as view
 from zun.api import utils as api_utils
 from zun.api import validation
@@ -36,6 +37,7 @@ from zun.common.i18n import _
 from zun.common import name_generator
 from zun.common.policies import container as policies
 from zun.common import policy
+from zun.common import quota
 from zun.common import utils
 import zun.conf
 from zun.network import model as network_model
@@ -46,6 +48,7 @@ from zun.volume import cinder_api as cinder
 
 CONF = zun.conf.CONF
 LOG = logging.getLogger(__name__)
+QUOTAS = quota.QUOTAS
 
 
 def check_policy_on_container(container, action):
@@ -73,7 +76,7 @@ class ContainerCollection(collection.Collection):
         context = pecan.request.context
         collection = ContainerCollection()
         collection.containers = \
-            [view.format_container(context, url, p.as_dict())
+            [view.format_container(context, url, p)
              for p in rpc_containers]
         collection.next = collection.get_next(limit, url=url, **kwargs)
         return collection
@@ -82,32 +85,6 @@ class ContainerCollection(collection.Collection):
 class ContainersActionsController(base.Controller):
     """Controller for Container Actions."""
 
-    def __init__(self):
-        super(ContainersActionsController, self).__init__()
-        self._action_keys = ['action', 'container_uuid', 'request_id',
-                             'user_id', 'project_id', 'start_time',
-                             'message']
-        self._event_keys = ['event', 'start_time', 'finish_time', 'result',
-                            'traceback']
-
-    def _format_action(self, action_raw):
-        action = {}
-        action_dict = action_raw.as_dict()
-        for key in self._action_keys:
-            action[key] = action_dict.get(key)
-        return action
-
-    def _format_event(self, event_raw, show_traceback=False):
-        event = {}
-        event_dict = event_raw.as_dict()
-        for key in self._event_keys:
-            # By default, non-admins are not allowed to see traceback details.
-            if key == 'traceback' and not show_traceback:
-                event['traceback'] = None
-                continue
-            event[key] = event_dict.get(key)
-        return event
-
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def get_all(self, container_ident, **kwargs):
@@ -118,7 +95,7 @@ class ContainersActionsController(base.Controller):
         container = utils.get_container(container_ident)
         actions_raw = objects.ContainerAction.get_by_container_uuid(
             context, container.uuid)
-        actions = [self._format_action(action) for action in actions_raw]
+        actions = [actions_view.format_action(a) for a in actions_raw]
 
         return {"containerActions": actions}
 
@@ -138,11 +115,7 @@ class ContainersActionsController(base.Controller):
             raise exception.ResourceNotFound(name="Action", id=request_ident)
 
         action_id = action.id
-        if CONF.database.backend == 'etcd':
-            # etcd using action.uuid get the unique action instead of action.id
-            action_id = action.uuid
-
-        action = self._format_action(action)
+        action = actions_view.format_action(action)
         show_traceback = False
         if policy.enforce(context, "container:action:events",
                           do_raise=False, action="container:action:events"):
@@ -150,7 +123,7 @@ class ContainersActionsController(base.Controller):
 
         events_raw = objects.ContainerActionEvent.get_by_action(context,
                                                                 action_id)
-        action['events'] = [self._format_event(evt, show_traceback)
+        action['events'] = [actions_view.format_event(evt, show_traceback)
                             for evt in events_raw]
         return action
 
@@ -261,11 +234,11 @@ class ContainersController(base.Controller):
             compute_api = pecan.request.compute_api
             try:
                 container = compute_api.container_show(context, container)
-            except exception.ContainerHostNotUp:
-                raise exception.ServerNotUsable
+            except exception.ContainerHostNotUp as e:
+                LOG.error("Failed to get container details: %s", str(e))
 
         return view.format_container(context, pecan.request.host_url,
-                                     container.as_dict())
+                                     container)
 
     def _generate_name_for_container(self):
         """Generate a random name like: zeta-22-container."""
@@ -291,7 +264,19 @@ class ContainersController(base.Controller):
 
         return self._do_post(run, **container_dict)
 
-    @base.Controller.api_version("1.20")  # noqa
+    @base.Controller.api_version("1.20", "1.35")  # noqa
+    @pecan.expose('json')
+    @api_utils.enforce_content_types(['application/json'])
+    @exception.wrap_pecan_controller_exception
+    @validation.validate_query_param(pecan.request, schema.query_param_create)
+    @validation.validated(schema.container_create)
+    def post(self, run=False, **container_dict):
+        interactive = container_dict.get('interactive')
+        if interactive is not None:
+            container_dict['tty'] = interactive
+        return self._do_post(run, **container_dict)
+
+    @base.Controller.api_version("1.36")  # noqa
     @pecan.expose('json')
     @api_utils.enforce_content_types(['application/json'])
     @exception.wrap_pecan_controller_exception
@@ -329,6 +314,9 @@ class ContainersController(base.Controller):
             raise exception.InvalidValue(_('Valid run or interactive '
                                            'values are: %s') % bools)
 
+        # Check container quotas
+        self._check_container_quotas(context, container_dict)
+
         auto_remove = container_dict.pop('auto_remove', None)
         if auto_remove is not None:
             api_utils.version_check('auto_remove', '1.3')
@@ -368,7 +356,8 @@ class ContainersController(base.Controller):
         if mounts:
             api_utils.version_check('mounts', '1.11')
 
-        requested_volumes = self._build_requested_volumes(context, mounts)
+        cpu_policy = container_dict.pop('cpu_policy', None)
+        container_dict['cpu_policy'] = cpu_policy
 
         privileged = container_dict.pop('privileged', None)
         if privileged is not None:
@@ -385,9 +374,13 @@ class ContainersController(base.Controller):
                                                'are: %s') % bools)
 
         # Valiadtion accepts 'None' so need to convert it to None
-        if container_dict.get('image_driver'):
-            container_dict['image_driver'] = api_utils.string_or_none(
-                container_dict.get('image_driver'))
+        container_dict['image_driver'] = api_utils.string_or_none(
+            container_dict.get('image_driver'))
+        if not container_dict['image_driver']:
+            container_dict['image_driver'] = CONF.default_image_driver
+        if container_dict.get('image_pull_policy'):
+            policy.enforce(context, "container:create:image_pull_policy",
+                           action="container:create:image_pull_policy")
 
         container_dict['project_id'] = context.project_id
         container_dict['user_id'] = context.user_id
@@ -398,6 +391,18 @@ class ContainersController(base.Controller):
         if container_dict.get('restart_policy'):
             utils.check_for_restart_policy(container_dict)
 
+        exposed_ports = container_dict.pop('exposed_ports', None)
+        if exposed_ports is not None:
+            api_utils.version_check('exposed_ports', '1.24')
+            exposed_ports = utils.build_exposed_ports(exposed_ports)
+            container_dict['exposed_ports'] = exposed_ports
+
+        registry = container_dict.pop('registry', None)
+        if registry:
+            api_utils.version_check('registry', '1.31')
+            registry = utils.get_registry(registry)
+            container_dict['registry_id'] = registry.id
+
         container_dict['status'] = consts.CREATING
         extra_spec = {}
         extra_spec['hints'] = container_dict.get('hints', None)
@@ -410,7 +415,8 @@ class ContainersController(base.Controller):
         kwargs = {}
         kwargs['extra_spec'] = extra_spec
         kwargs['requested_networks'] = requested_networks
-        kwargs['requested_volumes'] = requested_volumes
+        kwargs['requested_volumes'] = (
+            self._build_requested_volumes(context, new_container, mounts))
         if pci_req.requests:
             kwargs['pci_requests'] = pci_req
         kwargs['run'] = run
@@ -420,7 +426,48 @@ class ContainersController(base.Controller):
                                                  new_container.uuid)
         pecan.response.status = 202
         return view.format_container(context, pecan.request.host_url,
-                                     new_container.as_dict())
+                                     new_container)
+
+    def _check_container_quotas(self, context, container_delta_dict,
+                                update_container=False):
+        deltas = {
+            'containers': 0 if update_container else 1,
+            'cpu': container_delta_dict.get('cpu', 0),
+            'memory': container_delta_dict.get('memory', 0),
+            'disk': container_delta_dict.get('disk', 0)
+        }
+
+        def _check_deltas(context, deltas):
+            """Check usage deltas against quota limits.
+
+            This does QUOTAS.count() followed by a QUOTAS.limit_check()
+            using the provided deltas.
+
+            :param context: The request context, for access check.
+            :param deltas: A dict of {resource_name: delta, ...} to check
+                           against the quota limits.
+            """
+            check_kwargs = {}
+            count_as_dict = {}
+            project_id = context.project_id
+            for res_name, res_delta in deltas.items():
+                # TODO(kiennt): Apply count_as_dict method, query count usage
+                #               once rather than count each resource.
+                count_as_dict[res_name] = QUOTAS.count(context, res_name,
+                                                       project_id)
+                total = None
+                try:
+                    if isinstance(count_as_dict[res_name], six.integer_types):
+                        total = count_as_dict[res_name] + int(res_delta)
+                    else:
+                        total = float(count_as_dict[res_name]) + \
+                            float(res_delta)
+                except TypeError as e:
+                    raise e
+                check_kwargs[res_name] = total
+            QUOTAS.limit_check(context, project_id, **check_kwargs)
+
+        _check_deltas(context, deltas)
 
     def _set_default_resource_limit(self, container_dict):
         # NOTE(kiennt): Default disk size will be set later.
@@ -505,28 +552,40 @@ class ContainersController(base.Controller):
         phynet_name = net.get('provider:physical_network')
         return phynet_name
 
-    def _build_requested_volumes(self, context, mounts):
-        # NOTE(hongbin): We assume cinder is the only volume provider here.
-        # The logic needs to be re-visited if a second volume provider
-        # (i.e. Manila) is introduced.
+    def _build_requested_volumes(self, context, container, mounts):
         cinder_api = cinder.CinderAPI(context)
-        requested_volumes = []
+        requested_volumes = {container.uuid: []}
         for mount in mounts:
-            if mount.get('source'):
-                volume = cinder_api.search_volume(mount['source'])
-                auto_remove = False
-            else:
-                volume = cinder_api.create_volume(mount['size'])
-                auto_remove = True
-            cinder_api.ensure_volume_usable(volume)
-            volmapp = objects.VolumeMapping(
-                context,
-                volume_id=volume.id, volume_provider='cinder',
-                container_path=mount['destination'],
-                user_id=context.user_id,
-                project_id=context.project_id,
-                auto_remove=auto_remove)
-            requested_volumes.append(volmapp)
+            volume_dict = {
+                'cinder_volume_id': None,
+                'container_path': None,
+                'auto_remove': False,
+                'contents': None,
+                'user_id': context.user_id,
+                'project_id': context.project_id,
+            }
+            volume_type = mount.get('type', 'volume')
+            if volume_type == 'volume':
+                if mount.get('source'):
+                    volume = cinder_api.search_volume(mount['source'])
+                    cinder_api.ensure_volume_usable(volume)
+                    volume_dict['cinder_volume_id'] = volume.id
+                    volume_dict['container_path'] = mount['destination']
+                    volume_dict['volume_provider'] = 'cinder'
+                elif mount.get('size'):
+                    volume = cinder_api.create_volume(mount['size'])
+                    cinder_api.ensure_volume_usable(volume)
+                    volume_dict['cinder_volume_id'] = volume.id
+                    volume_dict['container_path'] = mount['destination']
+                    volume_dict['volume_provider'] = 'cinder'
+                    volume_dict['auto_remove'] = True
+            elif volume_type == 'bind':
+                volume_dict['contents'] = mount.pop('source', '')
+                volume_dict['container_path'] = mount['destination']
+                volume_dict['volume_provider'] = 'local'
+
+            volmapp = objects.VolumeMapping(context, **volume_dict)
+            requested_volumes[container.uuid].append(volmapp)
 
         return requested_volumes
 
@@ -612,25 +671,33 @@ class ContainersController(base.Controller):
         :param patch: a json PATCH document to apply to this container.
         """
         container = utils.get_container(container_ident)
+        context = pecan.request.context
+        container_deltas = {}
         check_policy_on_container(container.as_dict(), "container:update")
         utils.validate_container_state(container, 'update')
         if 'memory' in patch:
+            container_deltas['memory'] = \
+                int(patch['memory']) - int(container.memory)
             patch['memory'] = str(patch['memory'])
         if 'cpu' in patch:
             patch['cpu'] = float(patch['cpu'])
+            container_deltas['cpu'] = patch['cpu'] - container.cpu
         if 'name' in patch:
             patch['name'] = str(patch['name'])
-        context = pecan.request.context
+
         if 'memory' not in patch and 'cpu' not in patch:
             for field, patch_val in patch.items():
                 if getattr(container, field) != patch_val:
                     setattr(container, field, patch_val)
             container.save(context)
         else:
+            # Check container quotas
+            self._check_container_quotas(context, container_deltas,
+                                         update_container=True)
             compute_api = pecan.request.compute_api
             container = compute_api.container_update(context, container, patch)
         return view.format_container(context, pecan.request.host_url,
-                                     container.as_dict())
+                                     container)
 
     @base.Controller.api_version("1.1", "1.13")
     @pecan.expose('json')
@@ -651,7 +718,7 @@ class ContainersController(base.Controller):
         context = pecan.request.context
         container.save(context)
         return view.format_container(context, pecan.request.host_url,
-                                     container.as_dict())
+                                     container)
 
     @base.Controller.api_version("1.19")
     @pecan.expose('json')
@@ -676,7 +743,7 @@ class ContainersController(base.Controller):
         compute_api.resize_container(context, container, kwargs)
         pecan.response.status = 202
         return view.format_container(context, pecan.request.host_url,
-                                     container.as_dict())
+                                     container)
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
@@ -744,9 +811,12 @@ class ContainersController(base.Controller):
             container.image_driver = kwargs.get('image_driver')
         LOG.debug('Calling compute.container_rebuild with %s',
                   container.uuid)
+        run = True if container.status == consts.RUNNING else False
         context = pecan.request.context
+        container.status = consts.REBUILDING
+        container.save(context)
         compute_api = pecan.request.compute_api
-        compute_api.container_rebuild(context, container)
+        compute_api.container_rebuild(context, container, run)
         pecan.response.status = 202
 
     @pecan.expose('json')
@@ -998,6 +1068,7 @@ class ContainersController(base.Controller):
         compute_api = pecan.request.compute_api
         return compute_api.container_top(context, container, ps_args)
 
+    @base.Controller.api_version("1.1", "1.24")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def get_archive(self, container_ident, **kwargs):
@@ -1007,6 +1078,23 @@ class ContainersController(base.Controller):
         form of a tar archive.
         :param container_ident: UUID or Name of a container.
         """
+        kwargs['encode_data'] = False
+        return self._get_archive(container_ident, **kwargs)
+
+    @base.Controller.api_version("1.25")  # noqa
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def get_archive(self, container_ident, **kwargs):
+        """Retrieve a file/folder from a container
+
+        Retrieve a file or folder from a container in the
+        form of a tar archive.
+        :param container_ident: UUID or Name of a container.
+        """
+        kwargs['encode_data'] = True
+        return self._get_archive(container_ident, **kwargs)
+
+    def _get_archive(self, container_ident, **kwargs):
         container = utils.get_container(container_ident)
         check_policy_on_container(container.as_dict(), "container:get_archive")
         utils.validate_container_state(container, 'get_archive')
@@ -1016,9 +1104,10 @@ class ContainersController(base.Controller):
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         data, stat = compute_api.container_get_archive(
-            context, container, kwargs['path'])
+            context, container, kwargs['path'], kwargs['encode_data'])
         return {"data": data, "stat": stat}
 
+    @base.Controller.api_version("1.1", "1.24")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def put_archive(self, container_ident, **kwargs):
@@ -1028,6 +1117,23 @@ class ContainersController(base.Controller):
         a tar archive as source.
         :param container_ident: UUID or Name of a container.
         """
+        kwargs['decode_data'] = False
+        self._put_archive(container_ident, **kwargs)
+
+    @base.Controller.api_version("1.25")  # noqa
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def put_archive(self, container_ident, **kwargs):
+        """Insert a file/folder to container.
+
+        Insert a file or folder to an existing container using
+        a tar archive as source.
+        :param container_ident: UUID or Name of a container.
+        """
+        kwargs['decode_data'] = True
+        self._put_archive(container_ident, **kwargs)
+
+    def _put_archive(self, container_ident, **kwargs):
         container = utils.get_container(container_ident)
         check_policy_on_container(container.as_dict(), "container:put_archive")
         utils.validate_container_state(container, 'put_archive')
@@ -1036,8 +1142,9 @@ class ContainersController(base.Controller):
                   {'uuid': container.uuid, 'path': kwargs['path']})
         context = pecan.request.context
         compute_api = pecan.request.compute_api
-        compute_api.container_put_archive(context, container,
-                                          kwargs['path'], kwargs['data'])
+        compute_api.container_put_archive(
+            context, container, kwargs['path'], kwargs['data'],
+            kwargs['decode_data'])
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
@@ -1091,9 +1198,23 @@ class ContainersController(base.Controller):
         if kwargs.get('port'):
             port = neutron_api.get_neutron_port(kwargs['port'])
             net_id = port['network_id']
+            if (net_id not in container.addresses or
+                    port['id'] not in [a['port']
+                                       for a in container.addresses[net_id]]):
+                raise exception.Invalid(_(
+                    "Port '%(port)s' is not attached to container "
+                    "'%(container)s'.") %
+                    {"port": kwargs.get('port'),
+                     "container": container_ident})
         else:
             network = neutron_api.get_neutron_network(kwargs.get('network'))
             net_id = network['id']
+            if net_id not in container.addresses:
+                raise exception.Invalid(_(
+                    "Network '%(network)s' is not attached to container "
+                    "'%(container)s'.") %
+                    {"network": kwargs.get('network'),
+                     "container": container_ident})
         compute_api.network_detach(context, container, net_id)
         pecan.response.status = 202
 
@@ -1113,6 +1234,20 @@ class ContainersController(base.Controller):
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         requested_networks = utils.build_requested_networks(context, [kwargs])
+        if requested_networks[0]['network'] in container.addresses:
+            if kwargs.get('port'):
+                raise exception.Invalid(_(
+                    "Cannot attach port '%(port)s' to container "
+                    "'%(container)s' because another port on the "
+                    "same network is already attached to this container.") %
+                    {"port": kwargs.get('port'),
+                     "container": container_ident})
+            else:
+                raise exception.Invalid(_(
+                    "Network '%(network)s' is already connected to "
+                    "container '%(container)s'.") %
+                    {"network": kwargs.get('network'),
+                     "container": container_ident})
         compute_api.network_attach(context, container, requested_networks[0])
 
     @base.Controller.api_version("1.13", "1.17")
