@@ -24,6 +24,7 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import psutil
 import six
+import tenacity
 
 from zun.common import consts
 from zun.common import exception
@@ -107,6 +108,9 @@ def wrap_docker_error(function):
 class DockerDriver(driver.ContainerDriver):
     """Implementation of container drivers for Docker."""
 
+    # TODO(hongbin): define a list of capabilities of this driver.
+    capabilities = {}
+
     def __init__(self):
         super(DockerDriver, self).__init__()
         self._host = host.Host()
@@ -262,7 +266,7 @@ class DockerDriver(driver.ContainerDriver):
                 'environment': container.environment,
                 'working_dir': container.workdir,
                 'labels': container.labels,
-                'tty': container.interactive,
+                'tty': container.tty,
                 'stdin_open': container.interactive,
                 'hostname': container.hostname,
             }
@@ -280,17 +284,17 @@ class DockerDriver(driver.ContainerDriver):
             host_config['runtime'] = runtime
             host_config['binds'] = binds
             kwargs['volumes'] = [b['bind'] for b in binds.values()]
-            self._process_exposed_ports(network_api.neutron_api, container)
+            self._process_exposed_ports(network_api.neutron_api, container,
+                                        kwargs)
             self._process_networking_config(
                 context, container, requested_networks, host_config,
                 kwargs, docker)
             if container.auto_remove:
                 host_config['auto_remove'] = container.auto_remove
-            if container.memory is not None:
+            if self._should_limit_memory(container):
                 host_config['mem_limit'] = str(container.memory) + 'M'
-            if container.cpu is not None:
-                host_config['cpu_quota'] = int(100000 * container.cpu)
-                host_config['cpu_period'] = 100000
+            if self._should_limit_cpu(container):
+                host_config['cpu_shares'] = int(1024 * container.cpu)
             if container.restart_policy:
                 count = int(container.restart_policy['MaximumRetryCount'])
                 name = container.restart_policy['Name']
@@ -329,10 +333,18 @@ class DockerDriver(driver.ContainerDriver):
             container.addresses = addresses
 
             response = docker.inspect_container(container.container_id)
-            self._populate_container(container, response)
+            self._populate_container(container, response, force=True)
             container.save(context)
             return container
 
+    def _should_limit_memory(self, container):
+        return (container.memory is not None and
+                not isinstance(container, objects.Capsule))
+
+    def _should_limit_cpu(self, container):
+        return (container.cpu is not None and
+                not isinstance(container, objects.Capsule))
+
     def _is_runtime_supported(self):
         return float(CONF.docker.docker_remote_api_version) >= 1.26
 
@@ -342,15 +354,30 @@ class DockerDriver(driver.ContainerDriver):
     def get_host_default_base_size(self):
         return self.base_device_size
 
-    def _process_exposed_ports(self, neutron_api, container):
-        if not container.exposed_ports:
+    def _process_exposed_ports(self, neutron_api, container, kwargs):
+        exposed_ports = {}
+        if isinstance(container, objects.Container):
+            exposed_ports.update(container.exposed_ports or {})
+        if isinstance(container, objects.Capsule):
+            for container in (container.init_containers +
+                              container.containers):
+                exposed_ports.update(container.exposed_ports or {})
+
+        if not exposed_ports:
             return
 
+        # process security group
         secgroup_name = self._get_secgorup_name(container.uuid)
         secgroup_id = neutron_api.create_security_group({'security_group': {
             "name": secgroup_name}})['security_group']['id']
-        neutron_api.expose_ports(secgroup_id, container.exposed_ports)
+        neutron_api.expose_ports(secgroup_id, exposed_ports)
         container.security_groups = [secgroup_id]
+        # process kwargs on creating the docker container
+        ports = []
+        for port in exposed_ports:
+            port, proto = port.split('/')
+            ports.append((port, proto))
+        kwargs['ports'] = ports
 
     def _process_networking_config(self, context, container,
                                    requested_networks, host_config,
@@ -450,7 +477,14 @@ class DockerDriver(driver.ContainerDriver):
                 container, docker_net, neutron_network_id=neutron_net)
 
     def _cleanup_exposed_ports(self, neutron_api, container):
-        if not container.exposed_ports:
+        exposed_ports = {}
+        if isinstance(container, objects.Container):
+            exposed_ports.update(container.exposed_ports or {})
+        if isinstance(container, objects.Capsule):
+            for container in (container.init_containers +
+                              container.containers):
+                exposed_ports.update(container.exposed_ports or {})
+        if not exposed_ports:
             return
 
         try:
@@ -618,9 +652,9 @@ class DockerDriver(driver.ContainerDriver):
             return '{} seconds'.format(time_dict['seconds'])
         return
 
-    def _populate_container(self, container, response):
+    def _populate_container(self, container, response, force=False):
         state = response.get('State')
-        self._populate_container_state(container, state)
+        self._populate_container_state(container, state, force)
 
         config = response.get('Config')
         if config:
@@ -631,7 +665,19 @@ class DockerDriver(driver.ContainerDriver):
         if hostconfig:
             container.runtime = hostconfig.get('Runtime')
 
-    def _populate_container_state(self, container, state):
+    def _populate_container_state(self, container, state, force):
+        if container.task_state and not force:
+            # NOTE(hongbin): we don't want to populate container state
+            # if another thread is performing task on this container.
+            # In this case, task_state will be assigned (which means there is a
+            # task performing on this container) and 'force' will be set to
+            # False. For example, if this method is called by create,
+            # 'force' will be set to True to force refreshing the state.
+            # If this method is called by list or show,
+            # 'force' will be set to False, in which case we skip
+            # refreshing the state if there is a task on this container.
+            return
+
         if not state:
             LOG.warning('Receive unexpected state from docker: %s', state)
             container.status = consts.UNKNOWN
@@ -868,8 +914,7 @@ class DockerDriver(driver.ContainerDriver):
             args['memswap_limit'] = CONF.default_memory_swap
         cpu = patch.get('cpu')
         if cpu is not None:
-            args['cpu_quota'] = int(100000 * cpu)
-            args['cpu_period'] = 100000
+            args['cpu_shares'] = int(1024 * cpu)
 
         with docker_utils.docker_client() as docker:
             return docker.update_container(container.container_id, **args)
@@ -1084,26 +1129,10 @@ class DockerDriver(driver.ContainerDriver):
             # give another try with system root
             disk_usage = psutil.disk_usage('/')
         total_disk = disk_usage.total / 1024 ** 3
-        return int(total_disk * (1 - CONF.compute.reserve_disk_for_image))
-
-    def get_cpu_used(self):
-        cpu_used = 0
-        with docker_utils.docker_client() as docker:
-            containers = docker.containers()
-            for container in containers:
-                cnt_id = container['Id']
-                # Fixme: if there is a way to get all container inspect info
-                # for one call only?
-                inspect = docker.inspect_container(cnt_id)
-                cpu_period = inspect['HostConfig']['CpuPeriod']
-                cpu_quota = inspect['HostConfig']['CpuQuota']
-                if cpu_period and cpu_quota:
-                    cpu_used += float(cpu_quota) / cpu_period
-                else:
-                    if 'NanoCpus' in inspect['HostConfig']:
-                        nanocpus = inspect['HostConfig']['NanoCpus']
-                        cpu_used += float(nanocpus) / 1e9
-            return cpu_used
+        # TODO(hongbin): deprecate reserve_disk_for_image in flavor of
+        # reserved_host_disk_mb
+        return (int(total_disk),
+                int(total_disk * CONF.compute.reserve_disk_for_image))
 
     def add_security_group(self, context, container, security_group):
 
@@ -1193,6 +1222,11 @@ class DockerDriver(driver.ContainerDriver):
         capsule = self.create(context, capsule, image, requested_networks,
                               requested_volumes)
         self.start(context, capsule)
+        for container in capsule.init_containers:
+            self._create_container_in_capsule(context, capsule, container,
+                                              requested_networks,
+                                              requested_volumes)
+            self._wait_for_init_container(context, container)
         for container in capsule.containers:
             self._create_container_in_capsule(context, capsule, container,
                                               requested_networks,
@@ -1229,7 +1263,7 @@ class DockerDriver(driver.ContainerDriver):
                 'environment': container.environment,
                 'working_dir': container.workdir,
                 'labels': container.labels,
-                'tty': container.interactive,
+                'tty': container.tty,
                 'stdin_open': container.interactive,
             }
 
@@ -1247,8 +1281,7 @@ class DockerDriver(driver.ContainerDriver):
             if container.memory is not None:
                 host_config['mem_limit'] = str(container.memory) + 'M'
             if container.cpu is not None:
-                host_config['cpu_quota'] = int(100000 * container.cpu)
-                host_config['cpu_period'] = 100000
+                host_config['cpu_shares'] = int(1024 * container.cpu)
             if container.restart_policy:
                 count = int(container.restart_policy['MaximumRetryCount'])
                 name = container.restart_policy['Name']
@@ -1281,9 +1314,29 @@ class DockerDriver(driver.ContainerDriver):
             docker.start(container.container_id)
 
             response = docker.inspect_container(container.container_id)
-            self._populate_container(container, response)
+            self._populate_container(container, response, force=True)
             container.save(context)
 
+    def _wait_for_init_container(self, context, container, timeout=3600):
+        def retry_if_result_is_false(result):
+            return result is False
+
+        def check_init_container_stopped():
+            status = self.show(context, container).status
+            if status == consts.STOPPED:
+                return True
+            elif status == consts.RUNNING:
+                return False
+            else:
+                raise exception.ZunException(
+                    _("Container has unexpected status: %s") % status)
+
+        r = tenacity.Retrying(
+            stop=tenacity.stop_after_delay(timeout),
+            wait=tenacity.wait_exponential(),
+            retry=tenacity.retry_if_result(retry_if_result_is_false))
+        r.call(check_init_container_stopped)
+
     def delete_capsule(self, context, capsule, force):
         for container in capsule.containers:
             self._delete_container_in_capsule(context, capsule, container,
