@@ -11,27 +11,54 @@
 #    under the License.
 
 from neutron_lib import constants as n_const
+from neutronclient.common import exceptions as n_exceptions
 from neutronclient.neutron import v2_0 as neutronv20
+from oslo_log import log as logging
 from oslo_utils import uuidutils
 
 from zun.common import clients
+from zun.common import context as zun_context
 from zun.common import exception
 from zun.common.i18n import _
 
 
+LOG = logging.getLogger(__name__)
+
+
 class NeutronAPI(object):
 
     def __init__(self, context):
         self.context = context
-        self.neutron = clients.OpenStackClients(self.context).neutron()
+        self.client = clients.OpenStackClients(self.context).neutron()
+        self.admin_client = None
 
     def __getattr__(self, key):
-        return getattr(self.neutron, key)
+        return getattr(self.client, key)
+
+    def _get_admin_client(self):
+        if self.admin_client is None:
+            context = zun_context.get_admin_context()
+            self.admin_client = clients.OpenStackClients(context).neutron()
+        return self.admin_client
+
+    def update_port(self, port, body=None, admin=False):
+        if admin:
+            client = self._get_admin_client()
+        else:
+            client = self.client
+        return client.update_port(port, body)
+
+    def create_port(self, body=None, admin=False):
+        if admin:
+            client = self._get_admin_client()
+        else:
+            client = self.client
+        return client.create_port(body)
 
     def find_resourceid_by_name_or_id(self, resource, name_or_id,
                                       project_id=None):
         return neutronv20.find_resourceid_by_name_or_id(
-            self.neutron, resource, name_or_id, project_id)
+            self.client, resource, name_or_id, project_id)
 
     def get_available_network(self):
         search_opts = {'tenant_id': self.context.project_id, 'shared': False}
@@ -46,9 +73,9 @@ class NeutronAPI(object):
 
     def get_neutron_network(self, network):
         if uuidutils.is_uuid_like(network):
-            networks = self.neutron.list_networks(id=network)['networks']
+            networks = self.list_networks(id=network)['networks']
         else:
-            networks = self.neutron.list_networks(name=network)['networks']
+            networks = self.list_networks(name=network)['networks']
 
         if len(networks) == 0:
             raise exception.NetworkNotFound(network=network)
@@ -98,3 +125,27 @@ class NeutronAPI(object):
         binding_vif_type = port.get('binding:vif_type')
         if binding_vif_type == 'binding_failed':
             raise exception.PortBindingFailed(port=port['id'])
+
+    def expose_ports(self, secgroup_id, ports):
+        for port in ports:
+            port, proto = port.split('/')
+            secgroup_rule = {
+                'security_group_id': secgroup_id,
+                'direction': 'ingress',
+                'port_range_min': port,
+                'port_range_max': port,
+                'protocol': proto,
+                'remote_ip_prefix': '0.0.0.0/0',
+            }
+
+            try:
+                self.create_security_group_rule({
+                    'security_group_rule': secgroup_rule})
+            except n_exceptions.NeutronClientException as ex:
+                LOG.error("Error happened during creating a "
+                          "Neutron security group "
+                          "rule: %s", ex)
+                self.delete_security_group(secgroup_id)
+                raise exception.ZunException(_(
+                    "Could not create required security group rules %s "
+                    "for setting up exported port.") % secgroup_rule)
