@@ -31,6 +31,10 @@ class TestAPI(base.TestCase):
 
     def setUp(self):
         super(TestAPI, self).setUp()
+        p = mock.patch('zun.scheduler.client.query.SchedulerClient')
+        p.start()
+        self.addCleanup(p.stop)
+
         self.compute_api = api.API(self.context)
         self.container = objects.Container(
             self.context, **utils.get_test_container())
@@ -45,6 +49,7 @@ class TestAPI(base.TestCase):
                               mock_image_search,
                               mock_container_create,
                               mock_record_action_start):
+        CONF.set_override('enable_image_validation', True, group="api")
         container = self.container
         container.status = consts.CREATING
         image_meta = mock.MagicMock()
@@ -66,6 +71,7 @@ class TestAPI(base.TestCase):
     def test_container_create_with_private_registry_image(
             self, mock_schedule_container, mock_image_search,
             mock_container_create, mock_record_action_start):
+        CONF.set_override('enable_image_validation', True, group="api")
         container = self.container
         container.image = 'myregistry.io/test-image'
         container.image_driver = 'docker'
@@ -103,6 +109,7 @@ class TestAPI(base.TestCase):
                                        mock_schedule_container,
                                        mock_image_search,
                                        mock_start, mock_cast):
+        CONF.set_override('enable_image_validation', True, group="api")
         container = self.container
         container.status = consts.CREATING
         mock_schedule_container.return_value = {'host': u'Centos',
